#ifndef _TEXTURE_LIST_H_
#define _TEXTURE_LIST_H_

#include "ofMain.h"


class ofTextureList : public ofTexture{

	public :
	
		//-------------------------------------------------------	
		ofTextureList(){
			bUseList = false;
		}
		
		//-------------------------------------------------------
		void setUseList(bool useList){
			bUseList = useList;
		}
		
		//-------------------------------------------------------		
		void draw(int x, int y, int w, int h){
				
			//width and height - currently not implemented in draw for displayList
			//would be a matter of using glScalef to scale the texture to the correct size

			if(bUseList){
				
				glPushMatrix();
					glTranslatef(x, y, 0);					
					glCallList(texList);
				glPopMatrix();
				
			}else{
							
				ofTexture::draw(x,y,w,h);
			}
		}
		
		//-------------------------------------------------------
		void setupDisplayList(int scaleW = 0, int scaleH = 0){
				
				if(scaleW == 0) scaleW = width;
				if(scaleH == 0) scaleH = height;
			
				GLint px0 = 0;
				GLint py0 = 0;
				GLint px1 = (GLint)scaleW;
				GLint py1 = (GLint)scaleH;

				if (bFlipTexture == true){
					GLint temp = py0;
					py0 = py1;
					py1 = temp;
				}
				
				GLfloat offsetw = 0;
				GLfloat offseth = 0;

				if (textureTarget == GL_TEXTURE_2D){
					offsetw = 1.0f/(tex_w);
					offseth = 1.0f/(tex_h);
				}			
				
				GLfloat tx0 = 0+offsetw;
				GLfloat ty0 = 0+offseth;
				GLfloat tx1 = tex_t - offsetw;
				GLfloat ty1 = tex_u - offseth;
								
				texList = glGenLists(1);
				glNewList(texList, GL_COMPILE);			
					glEnable(textureTarget);

					// probably not a great idea to put it inside of the
					// display list as it then makes a copy of the texture for every instance
					// however I see no fps difference with it in or out of the display list 
					// (granted I am using an nvidia 8800GT)
					glBindTexture( textureTarget, (GLuint)textureName[0] );					
		
						glBegin( GL_QUADS );
							glTexCoord2f(tx0,ty0);			glVertex3i(px0, py0,0);
							glTexCoord2f(tx1,ty0);			glVertex3i(px1, py0,0);
							glTexCoord2f(tx1,ty1);			glVertex3i(px1, py1,0);
							glTexCoord2f(tx0,ty1);			glVertex3i(px0, py1,0);
						glEnd();

					glDisable(textureTarget);
				glEndList();
				
				//we now will use our display list instead of the normal draw method
				bUseList = true;
		}
			
	protected:		
		
		int		texList;
		bool	bUseList;
						
}; 

#endif
